/*************************************************************
  Höhen Sensor
  
  Sensor: MS5611 (GY-63) oder BMP180(GY-68) konfigurierbar
  I2C Adrtesse 77h

  MS5611 ist wesentlich genauer

  MS5611:  24bit ADC, 4096 osr, best update rate: 50Hz
  BMP180:  19bit ADC, 8 osr, best update rate: 33Hz
  ------------------------------------------------------------
  
  Copyright (C) 2017 Thomas Lehmann
  
  Version history:
  1.00   19.06.2017  created
  1.01   01.07.2017  BMP180 hinzugefügt

***************************************************************/


#include <Wire.h>

#include "SensAlt.h"



// ====
// Init
// ====
bool SensorAlt::Init( uint8_t Id, uint8_t Typ )
{
  if( TmoRefresh )                                // bereits gestartet ?
    return( true );

  // Sensor vorhanden / ansprechbar ?
  if( Typ == ALTTYP_MS6511 )
    {
    Wire.begin();                                 // I2C als Master starten
    Wire.beginTransmission( MS5611_ADDRESS );     // Sensor andingsen
    if( Wire.endTransmission() )                  // 0 == korrekte Antwort
      return( false );                            // Sensor nicht angeschlossen
    SensMS5611 = new MS5611;
    SensMS5611->begin( MS5611_STANDARD );         // Auflösung STD für kurze Verzögerung (3mS je Abfrage)
    RefPressure = SensMS5611->readPressure();     // akt. Druck als Referenz für relative Höhe
    }
  else
    {
    #if _BMP180_  == 1
    SensBMP180 = new SFE_BMP180;
    if( SensBMP180->begin() == 0 )
      return( false );                            // Sensor nicht angeschlossen
    #endif
    }

  SensTyp = Typ;
  TmoRefresh = ALT_TMO_REFRESH + Id;              // 10mS Timer asynchron zu anderen Sensoren bedienen
  return( true );
}



// ==================================================
// zyklischer Aufruf aus Loop()
// BMP180 bedienen
// ==================================================
#if _BMP180_  == 1
void SensorAlt::Loop()
{
 uint8_t BmpDelay = 0;

  // Wartezeit beendet, Sensor jetzt abfragbar ?
  if( TmoBMP180 == 0L || TmoBMP180 > millis() )
    return;

  switch( StateBmp )
    {
    case BMP_START_TEMP:
      // ret 0 bei Fehler
      BmpDelay = SensBMP180->startTemperature();   // Tempmessung starten
      break;
    case BMP_GET_TEMP:
      // Read true temperature, Druckabfrage nur wenn Temp bekannt ist starten
      if( SensBMP180->getTemperature(TValTemperature) )
        BmpDelay = 10;
      break;

    // Temp muss vor Druckabfrage gelesen worden sein !!
    case BMP_START_ALT:
      // ret 0 bei Fehler
      BmpDelay = SensBMP180->startPressure( 3 );   // Druckmessung mit höchster Genauigkeit starten
      break;
    case BMP_GET_ALT:
      {
      double realPressure;
      // akt. Druck als Referenz für relative Höhe, Temp muss vorher gelesen worden sein !!
      // ret 0 bei Fehler
      if( SensBMP180->getPressure(realPressure, TValTemperature) )
        {
        if( RefPressure == 0 )                    // 1. Messung, Starthöhe als Referenz
          RefPressure = realPressure;
        // aktuelle relative Höhe ggü Start
        TValAltitudeRelativ = SensBMP180->altitude( realPressure, RefPressure );
        //BmpDelay = 0;                     // fertig
        }
      break;
      }
    }

  if( BmpDelay )                            // nächste Bedienung verzögern, Sensor OK
    {
    TmoBMP180 = millis() + BmpDelay;
    StateBmp++;                             // weiter gehts
    }
  else                                      // fertig oder Fehler, nach Verzg. wieder neu starten
    {
    StateBmp   = BMP_START_TEMP;
    TmoBMP180  = 0L;
    TmoRefresh = ALT_TMO_REFRESH;
    }
}
#endif

// ==========
// 10mS Timer
// ==========
void SensorAlt::Tim10mS()
{
  if( TmoRefresh == 0 )                        // Sensor nicht angeschlossen
    return;

  // Sensor aktiv ?
  if( --TmoRefresh == 0 )
    {
    TmoRefresh = ALT_TMO_REFRESH;

    if( SensTyp == ALTTYP_MS6511 )
      {
      // Read true temperature, Sensor liefert 29,47 wird gewandelt in 294 > 29.4
      TVal.TemperatureDez1 = SensMS5611->readTemperature() * 10;

      // Calculate altitude
      long  realPressure = SensMS5611->readPressure();
      // aktuelle relative Höhe ggü NN
      // AltitudeAbsolut = SensMS5611->getAltitude( realPressure );
      // aktuelle relative Höhe ggü Start
      TVal.AltitudeRelativ = SensMS5611->getAltitude( realPressure, RefPressure );
      }
    #if _BMP180_  == 1
    else  // BMP 180
      {
      TmoBMP180 = 1;                        // Temp + Druckabfrage starten
      }
    #endif
    }
}








