// **************************************************************************************************************
//
// GPS generelle Akt/Deaktivierung
// NTC Erkennung -273° > Störung
//
// Multisensor für Jeti EX Telemetrie
// per JetiBox aktivierbare Sensoren
//
// Arudino Libs unter: (x86)\Arduino\hardware\arduino\avr\libraries\
// externe Librarys:
// TiniGps++          für GPS Sensor
// AltSoftSerial      dito, Software Serielle
// JetiEx Lib         Bernd Wokoeck https://sourceforge.net/projects/jetiexsensorcpplib/
//
//
// wenn kein Telemetrieframe gesendet wird, sendet der Empfänger alle 2Sek. seine Kennung
// feReceiver Type   Duplex R9 EX   ~ff
// feReceiver Type   Duplex R14 EX  ~ff
// feReceiver Type   Duplex Rsat2 EX~ff
//
// Hardware includes: "Arduino\hardware\tools\avr\avr\include\avr\"
// Register/ISR ... für 328er: iom328p.h
//
//
// Typ  Port  Funktion
// -------------------
// Tele UART  Empfängeranbindung
// RPM  IRQ   Drehzahlmessung
// FUEL IRQ   Verbrauchsmessung per Flowmeter
// ALT  I2C   Höhenmessung
// TEMP I2C   Motortemperatur per IR Sensor
// ACC  I2C   Beschleunigungsmessung und Neigungswinkel
// GPS  SER   Positionsbestimmung, Softserial IRQ gesteuert
// TEMP ADC   Motortemperatur per NTC Widerstand
//
//
// (c) 2017 Th. Lehmann
// thomas@thls.de
//
// Historie
// --------
// 12.06.2017 V0.00 Erstellung
// 04.07.2017 V1.00 erste Version
// 30.10.2017 V1.00 RPM + Fuel Messung getestet und optimiert
// 02.11.2017 V1.00 Sensor Check, Init und RunTime Fehleranzeige (JB)
// 13.11.2017 V1.01 erste Version
// 17.11.2017 V1.02 NTC Tempsensor checked, jetzt eingestellt auf 1K Vorwiderstand und 10K NTC
// 21.11.2017 V1.03 GPS:Trip, gesamt zurückgelegte Strecke
// 23.11.2017 V1.04 IR Temperatursensor MXL90614
// 24.11.2017 V1.05 Tankvolume Autorücksetzen konfigurierbar
//                  Update to V1.05 JetiExSensor Lib from Sepp
//            V1.06 GPS: Trip und Gesamt Strecke (KM Stand)
// 30.11.2017 V1.06 Tankvolume Rest in Prozent, auf besonderen Wunsch von Alex F.
// 11.12.2017 V1.07 NTC und IR Temperatur Sensor auswählbar
// 
// **************************************************************************************************************

                   //1234567890123456
#define JB_TEXT_Z1  " XSensor by ThL"
#define JB_TEXT_Z2  "V1.07 11.12.2017"


#include "EEPROM.h"
//#include <avr/wdt.h>
// wdt_enable( WDTO_2S );                      // Watchdog aktivieren, Tmo 2Sek.
// wdt_reset();                                // Watchdog triggern


#include "Def.h"


#include "JetiExProtocol.h"
JetiExProtocol JetiEx;

// Sensoren
#include "SensRpm.h"
SensorRpm *SensRpm;

#include "SensFuel.h"
SensorFuel *SensFuel;

#include "SensAlt.h"
SensorAlt *SensAlt;

#include "SensTemp.h"
SensorTemp *SensTemp[CNT_SENS_TEMP];

#if _SENS_GPS_ == 1
#include "SensGPS.h"
SensorGps *SensGps;
#endif

#if _SENS_ACC_ == 1
#include "SensACC.h"
SensorAcc *SensAcc;
#endif



// LED Anzeige
// ===========                      // ein / aus Zeit in mS, je BIT 100mS max 1600mS
#define LED_OFF     0x0000          //   0 /    0  LED aus
#define LED_ON      0xffff          //   1 /    0  LED statisch an
#define LED_SLOW    0x00ff          // 800 /  800  LED blinkt langsam
#define LED_FAST    0x0f0f          // 400 /  400  LED blinkt schnell
#define LED_VFAST   0x3333          // 200 /  200  LED blinkt sehr schnell
#define LED_VVFAST  0x5555          // 100 /  100  LED blinkt sehr sehr schnell
#define LED_FLASH1  0x0001          // 100 / 1500  LED Flash
#define LED_FLASH2  0x0101          // 100 /  700  LED Flash
#define LED_FLASH3  0x1111          // 100 /  300  LED Flash
#define LED_FLASH4  0xfffb          // 1400/  200  LED Flash

#define LED_SILENT     LED_FLASH1   // Ruhebetrieb
#define LED_NO_SENS    LED_FLASH4   // kein Sensor aktiv
#define LED_SENS_INI   LED_VVFAST   // Sensor init Fehler
#define LED_SENS_ERR   LED_SLOW     // Sensor failure

static uint16_t LedState;           // akt. LED Blinkrhytmus


static uint8_t CntSensActiv;        // # aktiver Sensoren

enum
{
  ID_RPM = 1,
  ID_RPM_REVOL,
  ID_RPM_RUNTIME,

  ID_FUEL_VOL,
  ID_FUEL_VOL_PROZ,
  ID_FUEL_FLOW,
  ID_FUEL_TOT,

  ID_TEMP1,
  ID_TEMP2,
  ID_TEMP3,
  ID_TEMP4,
  ID_TEMP5,
  ID_TEMP6,
  ID_TEMP7,

  ID_ALT_REL,
  ID_ALT_TEMP,

  #if _SENS_GPS_ == 1
  ID_GPS_SATS,
  ID_GPS_LAT,
  ID_GPS_LON,
  ID_GPS_ALT,
  ID_GPS_SPEED,
  ID_GPS_SPEED_MAX,
  ID_GPS_HEAD,
  ID_GPS_COURSE,
  ID_GPS_DIST_TO,
  ID_GPS_DIST_TRIP,
  ID_GPS_DIST_TOT,
  #endif

  #if _SENS_ACC_ == 1
  ID_ACC_GFORCE_X,
  ID_ACC_GFORCE_Y,
  ID_ACC_GFORCE_Z,
  ID_ACC_PITCH,
  ID_ACC_ROLL,
  #endif

  ID_CNT
};
#if ID_CNT > 32                             // wegen Bit Masken nur 31 Sensoren (1-31)
Anzahl Sensoren !!!
#endif
#define SENS_MSK_DEF    0                   // kein Sensor aktiv
//#define SENS_MSK_DEF    (1<<ID_RPM)       // RPM Sensor ist immer aktiv


// max. 32 Sensoren
// Id von 1..255
// Name plus Unit < 20 characters
//  - Name < 20 characters inkl. Sensornamen (NAME_SENSOR)
//  - unit <  7 characters
// precision = 0 --> 0, precision = 1 --> 0.0, precision = 2 --> 0.00
// Wertebereich:
// TYPE_6b   int6_t  Data type 6b  (       -31        +31)
// TYPE_14b  int14_t Data type 14b (     -8191      +8191)
// TYPE_22b  int22_t Data type 22b (  -2097151   +2097151)
// TYPE_30b  int30_t Data type 30b (-536870911 +536870911) 
#define NAME_SENSOR "XSens"
JETISENSOR_CONST Sensors[] PROGMEM =
{
  // id               name               unit         data type             precision
  //       #20> XSens:12345678901234
  { ID_RPM,          "RPM",             "/min",       JetiSensor::TYPE_22b, 0 },
  { ID_RPM_REVOL,    "RPM.Revol tot",   "Mil",        JetiSensor::TYPE_22b, 2 }, // dez.2
  { ID_RPM_RUNTIME,  "RPM.Runtime",     "h",          JetiSensor::TYPE_22b, 2 }, // dez.2

  { ID_FUEL_VOL,     "FUEL.Volume",     "mL",         JetiSensor::TYPE_14b, 0 },
  { ID_FUEL_VOL_PROZ,"FUEL.Volume",     "%",          JetiSensor::TYPE_14b, 0 },
  { ID_FUEL_FLOW,    "FUEL.Flow",       "mL",         JetiSensor::TYPE_14b, 0 },
  { ID_FUEL_TOT,     "FUEL.Flow tot",   "L",          JetiSensor::TYPE_22b, 2 }, // dez.2

  { ID_TEMP1,        "Temp-1",          "\xB0\x43",   JetiSensor::TYPE_14b, 0 }, // °C
  { ID_TEMP2,        "Temp-2",          "\xB0\x43",   JetiSensor::TYPE_14b, 0 }, // °C
  { ID_TEMP3,        "Temp-3",          "\xB0\x43",   JetiSensor::TYPE_14b, 0 }, // °C
  { ID_TEMP4,        "Temp-4",          "\xB0\x43",   JetiSensor::TYPE_14b, 0 }, // °C
  { ID_TEMP5,        "Temp-5",          "\xB0\x43",   JetiSensor::TYPE_14b, 0 }, // °C
  { ID_TEMP6,        "Temp-6",          "\xB0\x43",   JetiSensor::TYPE_14b, 0 }, // °C
  { ID_TEMP7,        "Temp-7",          "\xB0\x43",   JetiSensor::TYPE_14b, 0 }, // °C
  #if CNT_SENS_TEMP != 7
  hier ist was faul
  #endif

  //       #20> XSens:12345678901234
  { ID_ALT_REL,      "ALT.Altit. rel",  "m",          JetiSensor::TYPE_14b, 0 },
  { ID_ALT_TEMP,     "ALT.Amb. temp",   "\xB0\x43",   JetiSensor::TYPE_14b, 1 }, // °C

  #if _SENS_GPS_ == 1
  { ID_GPS_SATS,     "GPS.Satellites",  "#",          JetiSensor::TYPE_14b, 0 },
  { ID_GPS_LAT,      "GPS.Latitude",    " ",          JetiSensor::TYPE_GPS, 0 },
  { ID_GPS_LON,      "GPS.Longitude",   " ",          JetiSensor::TYPE_GPS, 0 },
  { ID_GPS_ALT,      "GPS.Altitude",    "m",          JetiSensor::TYPE_14b, 0 },
  { ID_GPS_SPEED,    "GPS.Speed",       "km/h",       JetiSensor::TYPE_14b, 0 },
  { ID_GPS_SPEED_MAX,"GPS.Speed max.",  "km/h",       JetiSensor::TYPE_14b, 0 },
  { ID_GPS_HEAD,     "GPS.Heading",     "\xB0",       JetiSensor::TYPE_14b, 0 }, // °
  { ID_GPS_COURSE,   "GPS.Course to",   "\xB0",       JetiSensor::TYPE_14b, 0 }, // °
  { ID_GPS_DIST_TO,  "GPS.Dist to",     "m",          JetiSensor::TYPE_14b, 0 },
  { ID_GPS_DIST_TRIP,"GPS.Dist trip",   "m",          JetiSensor::TYPE_22b, 0 }, 
  { ID_GPS_DIST_TOT, "GPS.Dist total",  "km",         JetiSensor::TYPE_22b, 2 }, // dez.2
  #endif

  #if _SENS_ACC_ == 1
  { ID_ACC_GFORCE_X, "ACC.GForce X",    "G",          JetiSensor::TYPE_14b, 0 },
  { ID_ACC_GFORCE_Y, "ACC.GForce Y",    "G",          JetiSensor::TYPE_14b, 0 },
  { ID_ACC_GFORCE_Z, "ACC.GForce Z",    "G",          JetiSensor::TYPE_14b, 0 },
  { ID_ACC_PITCH,    "ACC.Pitch",       "\xB0",       JetiSensor::TYPE_14b, 0 },
  { ID_ACC_ROLL,     "ACC.Roll",        "\xB0",       JetiSensor::TYPE_14b, 0 },
  #endif

  { 0 }   // end of array
 /*
  { ID_VOLT,         "Voltage",         "V",          JetiSensor::TYPE_14b, 0 },
  { ID_AMPERE,       "Ampere",          "A",          JetiSensor::TYPE_14b, 0 },
  { ID_WATT,         "Power",           "W",          JetiSensor::TYPE_14b, 0 },
    { ID_VOLTAGE,    "Voltage",    "V",         JetiSensor::TYPE_14b, 1 },
    { ID_VOLTAGE,    "Voltage",    "A",         JetiSensor::TYPE_14b, 1 },

    { ID_CLIMB,      "Climb",      "m/s",       JetiSensor::TYPE_14b, 2 },
    { ID_DATE,       "Date",       " ",         JetiSensor::TYPE_DT,  0 },
    { ID_TIME,       "Time",       " ",         JetiSensor::TYPE_DT,  0 },
  */
};


// JetiBox + Telemetrieübertragung
// -------------------------------
static struct
  {
  uint32_t AvailMsk;                  // verfügbare + aktive Sensoren, BIT je Idx = ID_XX
  uint32_t ErrInit;                   // Fehler bei der Sensor Initialisierung
  int16_t  TmoRefresh;                // JB Tasten und Telewerte nur alle 100mS checken
    #define TMO_TELE_SND  (100/TIM_BASE)
  }Tele;


// Sensor Test aktiv, auch Flow Kalibrierung
// -----------------------------------------
struct
  {
  uint8_t  Id;                        // Sensor ID
  uint8_t  IdLast;
  uint32_t StartVal;                  // 0 = neue Messung
  uint16_t MessureVal;                // Differenz zum Start, Sensor meldet Änderung
  }SensorChk;




// Aufbau der Daten im EEProm
// max 1024Byte beim ATmega328
// ---------------------------
struct
  {
  struct
    {
    uint16_t Magic;                         // EEProm gültig, nur um bei Erstinbetriebnahme zu löschen
      #define EEP_MAGIC_NV    0x30a7        // 30.10.17
    uint16_t Align;

    // Summenwerte, löschen nur wenn vom User gewünscht
    uint32_t RpmRevolTot;                   // Gesamtdrehzahl
    uint32_t RpmRunTimeS;                   // Gesamt Betriebszeit in Sekunden
    
    uint32_t FuelFlowCntTank;               // # Pulse bei akt. Tank gemessen / entnommene Menge
    uint32_t FuelFlowCntTot;                // # Pulse gesamt entnommene Menge

    uint32_t GpsDistTot;                    // Gesamt zurückgelegte Strecke, KM Stand

    uint32_t Res[20];
    }NV;


  struct
    {
    uint16_t Magic;                         // EEProm gültig
      #define EEP_MAGIC_DATA  0x11c7        // 11.12.17
  
    uint32_t SensActivMsk;                  // BIT je aktiven Sensor (1-31)
    uint16_t Res;
  
    uint8_t  RpmCfgCntMag;                  // RPM, # Magnete, 0=einer / 1=zwei
    uint8_t  RpmCfgAccuracy;                // Ergebnis glätten, Genauigkeit n UPM, 0-100
      #define RPM_ACCURACY_DEF   10
      #define RPM_ACCURACY_MAX  100
  
    uint16_t FuelCfgVolume;                 // Tankvolumen in mL
      #define FUEL_VOLUME_DEF   750
    uint8_t  FuelCfgResAuto;                // 1 = auto Reset, 0 = manuell per Taster
    uint16_t FuelCfgFlowCnt1L;              // # Pulse / Liter
  
    uint8_t CfgSensTypAlt;                  // Sensortyp Altimeter
    uint8_t CfgSensTypTemp[CNT_SENS_TEMP];  // Sensortyp Temperatur
  
    // Kalibrierungswerte Accelerator / G-Force
    int8_t GForceCfgCalX;
    int8_t GForceCfgCalY;
    int8_t GForceCfgCalZ;
    int8_t GForceCfgCalPitch;
    int8_t GForceCfgCalRoll;
    }Data;
  }EEprom;

static bool EEpromSave;                     // EEProm Daten geändert > speichern

static uint8_t Signal;



// Funktionsprototypen
// -------------------
static void HandleTim( void );
static void EEpromRW( bool Write );
static void EEpromChk( void );
static void HandleJB( bool Refresh );
static uint8_t ConGetKey( void );



#if SOFT_TERM == 1
#include <AltSoftSerial.h>
AltSoftSerial *TermSoftSerial;
uint8_t TermBuff[20];
void TermPut( void *Buff );
void TermPutln( void *Buff );
#endif



#if HW_TERM == 1
int freeRam()
{
 extern int __heap_start, *__brkval;
 int v;
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval);
}
#endif

#include "JetiBox.h"



// Watchdog Time-out
// =================
/*
ISR( WDT_vect )
{
  WDTCSR = 0;                         // Watchdog aus, akt. installierter Bootloader macht das leider nicht, Update auf OptiBoot notwendig
  OUT_TGL( PIN_DBG );
  SoftReset();                        // Neustart
}
*/


// ====
// INIT
// ====
void setup( void )
{
#if SOFT_TERM == 1
  TermSoftSerial = new AltSoftSerial;          // serielle auf PIN 8 (RX) + 9 (TX)
  TermSoftSerial->begin( 9600 );
  delay(1000);
#endif

#if HW_TERM == 1
  Serial.begin(9600);

  TermPutTxt("\nXSensor Dbg\n");
  TermPutTxt("free RAM: ");
  TermPutln(freeRam(), DEC);
  TermPutln();
#endif

#if DEBUG_PIN == 1
  pinMode( PIN_DBG1, OUTPUT );
  digitalWrite( PIN_DBG1, HIGH );
  pinMode( PIN_DBG2, OUTPUT );
  digitalWrite( PIN_DBG2, HIGH );
#endif


  EEpromRW( false );                           // EEProm lesen
  pinMode( PIN_LED, OUTPUT );
  digitalWrite( PIN_LED, PIN_LED_ON );

//  analogReference( INTERNAL );

  // aktive Sensoren ermitteln und initialisieren
  // --------------------------------------------
  for( uint8_t Idx = 1; Idx < ID_CNT; Idx++ )
    {
    bool Avail;
    uint8_t i;

    if( !BIT_TST(EEprom.Data.SensActivMsk, Idx) )    // Sensor aktiv geschaltet ?
      {
      JetiEx.SetSensorActive( Idx, false, Sensors );  // nö, dann auch nicht an den Sender übermitteln, muss vor JetiEx.Start() aufgerufen werden
      continue;
      }
    switch ( Idx )
      {
      case ID_RPM:
      case ID_RPM_REVOL:
      case ID_RPM_RUNTIME:
        if( SensRpm == 0L )
          SensRpm = new SensorRpm();
        Avail = SensRpm->Init( &Signal, EEprom.Data.RpmCfgCntMag, EEprom.Data.RpmCfgAccuracy, EEprom.NV.RpmRevolTot, EEprom.NV.RpmRunTimeS );
        break;

      case ID_FUEL_VOL:
      case ID_FUEL_VOL_PROZ:
      case ID_FUEL_FLOW:
      case ID_FUEL_TOT:
        if( SensFuel == 0L )
          SensFuel = new SensorFuel();
        Avail = SensFuel->Init( &Signal, EEprom.Data.FuelCfgResAuto, EEprom.Data.FuelCfgVolume, EEprom.Data.FuelCfgFlowCnt1L, EEprom.NV.FuelFlowCntTank, EEprom.NV.FuelFlowCntTot );
        break;

      case ID_ALT_REL:
      case ID_ALT_TEMP:
        if( SensAlt == 0L )
          SensAlt = new SensorAlt();
        Avail = SensAlt->Init( Idx, EEprom.Data.CfgSensTypAlt );
        break;

      case ID_TEMP1 ... ID_TEMP7:
        i = Idx - ID_TEMP1;
        SensTemp[i] = new SensorTemp();
        Avail = SensTemp[i]->Init( Idx, EEprom.Data.CfgSensTypTemp[i], i );
        break;

      #if _SENS_GPS_ == 1
      // GPS Sensoren
      case ID_GPS_SATS:
      case ID_GPS_LAT:
      case ID_GPS_LON:
      case ID_GPS_ALT:
      case ID_GPS_SPEED:
      case ID_GPS_SPEED_MAX:
      case ID_GPS_HEAD:
      case ID_GPS_COURSE:
      case ID_GPS_DIST_TO:
      case ID_GPS_DIST_TRIP:
      case ID_GPS_DIST_TOT:
        if( SensGps == 0L )
          SensGps = new SensorGps();
        Avail = SensGps->Init( &Signal, Idx, EEprom.NV.GpsDistTot );
        break;
      #endif

      #if _SENS_ACC_ == 1
      // Acceleration G-Force
      case ID_ACC_GFORCE_X:
      case ID_ACC_GFORCE_Y:
      case ID_ACC_GFORCE_Z:
      case ID_ACC_PITCH:
      case ID_ACC_ROLL:
        if( SensAcc == 0L )
          SensAcc = new SensorAcc();
        Avail = SensAcc->Init( Idx, EEprom.Data.GForceCfgCalX, EEprom.Data.GForceCfgCalY, EEprom.Data.GForceCfgCalZ, EEprom.Data.GForceCfgCalPitch, EEprom.Data.GForceCfgCalRoll );
        break;
      #endif
      }

    CntSensActiv++;                                     // # aktivierter Sensoren
    if( Avail )
      BIT_SET( Tele.AvailMsk, Idx );
    else
      BIT_SET( Tele.ErrInit, Idx );
    }


  // Jeti EX Protokoll aktivieren, Sensor und Sensorwerte anlegen
  JetiEx.SetDeviceId( 0x76, 0x32 );                  // 0x3276
  JetiEx.Start( NAME_SENSOR, Sensors, JetiExProtocol::SERIAL2 );
  JetiEx.SetJetiboxText( JetiExProtocol::LINE1, JB_TEXT_Z1 );
  JetiEx.SetJetiboxText( JetiExProtocol::LINE2, JB_TEXT_Z2 );
  //Tele.TmoRefresh = 1;                               // JB Tasten und Telewerte direkt mal checken

//  wdt_enable( WDTO_2S );                      // Watchdog aktivieren, Tmo 2Sek.
//  WDTCSR |= (1<<WDIE);                        // Timeout IRQ aktivieren

/*
#define WDTCSR _SFR_MEM8(0x60)
#define WDP0 0
#define WDP1 1
#define WDP2 2
#define WDE 3
#define WDCE 4
#define WDP3 5
#define WDIE 6
#define WDIF 7
*/
}


void loop( void )
{
 static uint32_t Tim;

//  wdt_reset();                                // Watchdog triggern

  #if DEBUG_SCEDULES == 1
  static long SceduleCnt;
  SceduleCnt++;
  #endif

  #if _BMP180_  == 1
  if( SensAlt )
    SensAlt->Loop();
  #endif

  #if _SENS_GPS_ == 1
  if( SensGps )
    SensGps->Loop();
  #endif

  // 10mS Timer generieren
  // ---------------------
  if ( Tim <= millis() )
    {
    Tim = millis() + TIM_BASE;
    HandleTim();                              // 10mS Timer bedienen

    #if DEBUG_SCEDULES == 1
    static uint16_t t100;
    if ( ++t100 > 100 )
      {
      t100 = 0;
      TermPutTxt("SceduleCnt: ");
      TermPutln(SceduleCnt);
      SceduleCnt = 0;
      }
    #endif
    }


  // anstehende Events bewerten
  // --------------------------
  /*  if( Signal & SIG_RPM )
      {
      Signal &= ~SIG_RPM;
      }
  */
}



// ===================
// 10mS Timer bedienen
// ===================
void HandleTim( void )
{
 static uint8_t TmoLED, LedShift, Idx;
 static uint16_t Save2NVTmo;
  #define SAVE_2NV_TMO  (5000/TIM_BASE)
 bool SensFailure = false;

  // Sensor Timer bedienen
  // ---------------------
  if( SensRpm )
    SensRpm->Tim10mS();                       // Drehzahl
  if( SensFuel )
    SensFuel->Tim10mS();                      // Durchfluss
  if( SensAlt )
    SensAlt->Tim10mS();                       // Höhe
  // Temperatur
  for( Idx = 0; Idx < CNT_SENS_TEMP; Idx++ )
    if( SensTemp[Idx] )
      {
      SensTemp[Idx]->Tim10mS();
      SensFailure |= SensTemp[Idx]->Failure;  // irgendwas faul mit dem Sensor ?
      }
  #if _SENS_GPS_ == 1
  if( SensGps )
    {
    SensGps->Tim10mS();                       // GPS
    SensFailure |= SensGps->Failure;          // irgendwas faul im GPS ?
    }
  #endif
  #if _SENS_ACC_ == 1
  if( SensAcc )
    SensAcc->Tim10mS();                       // G-Force
  #endif

  //JetiEx.DoJetiSend();                      // Routine sendet nur alle 150mS
  JetiEx.Tim10mS();                           // JetiEx senden, nur alle 150mS einen Frame
  HandleJB( false );                          // JetiBox bedienen


  if( CntSensActiv == 0 )                     // kein einziger Sensor aktiv ? was soll denn der Quatsch ;)
    LedState = LED_NO_SENS;                   // evt. EEprom gelöscht
  else if( Tele.ErrInit )                     // Sensor init fehlgeschlagen
    LedState = LED_SENS_INI;
  else if( SensFailure )                      // hat ein Sensor ein Problem ?
    LedState = LED_SENS_ERR;                  // Sensor failure
  else
    LedState = LED_SILENT;                    // alles im grünen Bereich


  // LED Blinkrhytmus
  // ----------------
  if( ++TmoLED >= 10 )                        // je Tick 100mS
    {
    TmoLED = 0;
    digitalWrite( PIN_LED, (LedState >> LedShift) & 1 );
    LedShift++;
    LedShift &= 0x0f;                         // nur 16 BIT Blinken
    }


  // Summenwerte im EEProm sichern wenn sich nichts mehr ändert
  // ==========================================================
  if( Signal & SIG_SAVE2NV )                  // irgendein Sensor signalisiert das Änderungen vorliegen
    {
    Signal &= ~SIG_SAVE2NV;
    // wenn sich nichts mehr ändert wird sehr wahrscheinlich geich ausgeschaltet, vorher noch ins EEProm schreiben
    Save2NVTmo = SAVE_2NV_TMO;
    }

  if( Save2NVTmo && (--Save2NVTmo) == 0 )
    {
    // Gesamtdrehzahl, Betriebszeit
    if( SensRpm )
      {
      EEprom.NV.RpmRevolTot = SensRpm->NV.RevolTot; // # Umdrehungen total, inkl. vorheriger Speicherung
      EEprom.NV.RpmRunTimeS = SensRpm->NV.RunTimeS; // Betriebszeit in Sek.
      }
    // Restvolumen, Durchfluss gesamt
    if( SensFuel )
      {
      EEprom.NV.FuelFlowCntTank  = SensFuel->NV.FlowCntTank;  // # Pulse entnommen bei akt. Tankfüllung
      EEprom.NV.FuelFlowCntTot   = SensFuel->NV.FlowCntTot;   // # Pulse entnommen gesamt
      }
    #if _SENS_GPS_ == 1
    // zurückgelegte Strecke gesamt
    if( SensGps )
      EEprom.NV.GpsDistTot = SensGps->NV.DistTot;
    #endif

    EEpromRW( true );
    }




  // Sensor Test aktiv, auch Flow Kalibrierung
  // -----------------------------------------
  if( SensorChk.Id != SensorChk.IdLast )
    {
    SensorChk.IdLast = SensorChk.Id;
    SensorChk.StartVal = 0L;
    }
  if( SensorChk.Id )
    {
    uint32_t Val;

    if( BIT_TST(Tele.AvailMsk, SensorChk.Id) )              // Sensor aktiv ?
      {
      if( SensorChk.Id == ID_RPM )                          // Hallgeber Test ?
        Val = SensRpm->NV.RevolTot;
      else if( SensorChk.Id == ID_FUEL_FLOW )               // Durchflußgeber Test ?
        Val = SensFuel->NV.FlowCntTot;
  //  else if( SensorChk.Id == ID_GPS )                     // seriellen Empfang testen ?
  //    Val = SensGps->Read_RxChCnt();
      }
    else
      Val = 0L;

    if( SensorChk.StartVal == 0L )                        // 1. Aufruf ?
      {
      SensorChk.StartVal = Val;
      SensorChk.MessureVal = 0L;
      HandleJB( true );                                   // Refresh der Anzeige auf 0
      }
    if( SensorChk.StartVal != Val )                       // Änderung, tut sich was ?
      {
      SensorChk.MessureVal = Val - SensorChk.StartVal;
      HandleJB( true );                                   // Refresh der Anzeige im akt. Menu
      }
    }

    
  // ---------------------------------
  // zykl. Refresh Telemetrie Sensoren
  // ---------------------------------
  //if( Tele.TmoRefreshTmo && (--Tele.TmoRefresh == 0) )
  if( --Tele.TmoRefresh <= 0 )
    {
    Tele.TmoRefresh = TMO_TELE_SND;           // delay bis zum nächsten Check


    // Drehzahl
    if( BIT_TST(Tele.AvailMsk, ID_RPM) )
      JetiEx.SetSensorValue( ID_RPM, SensRpm->TVal.Rpm );
    if( BIT_TST(Tele.AvailMsk, ID_RPM_REVOL) )
      JetiEx.SetSensorValue( ID_RPM_REVOL, SensRpm->TVal.RevolTotDez2 );
    if( BIT_TST(Tele.AvailMsk, ID_RPM_RUNTIME) )          // Anzeige in Stunden mit 2 dezimal Stellen, 100Min > 1.40
      JetiEx.SetSensorValue( ID_RPM_RUNTIME, SensRpm->TVal.RunTimeDez2 );

    // Flowmeter
    // ---------
    if( BIT_TST(Tele.AvailMsk, ID_FUEL_VOL) )
      JetiEx.SetSensorValue( ID_FUEL_VOL, SensFuel->TVal.VolumeRemain );
    if( BIT_TST(Tele.AvailMsk, ID_FUEL_VOL_PROZ) )
        {
        float Proz = 0;
        if( SensFuel->TVal.VolumeRemain <= EEprom.Data.FuelCfgVolume )  // Überlauf, mehr entnommen als vorhanden ;)
          Proz = (100.0/EEprom.Data.FuelCfgVolume) * SensFuel->TVal.VolumeRemain;
        JetiEx.SetSensorValue( ID_FUEL_VOL_PROZ, Proz );
        }
    if( BIT_TST(Tele.AvailMsk, ID_FUEL_FLOW) )
      JetiEx.SetSensorValue( ID_FUEL_FLOW, SensFuel->TVal.Flow );
    if( BIT_TST(Tele.AvailMsk, ID_FUEL_TOT) )
      JetiEx.SetSensorValue( ID_FUEL_TOT, SensFuel->TVal.FlowTotDez2 );

    // Höhe
    // ----
    if( BIT_TST(Tele.AvailMsk, ID_ALT_REL) )
      JetiEx.SetSensorValue( ID_ALT_REL, SensAlt->TVal.AltitudeRelativ );
    if( BIT_TST(Tele.AvailMsk, ID_ALT_TEMP) )
      JetiEx.SetSensorValue( ID_ALT_TEMP, SensAlt->TVal.TemperatureDez1 );

    // Temperatur
    // ----------
    for( Idx = 0; Idx < CNT_SENS_TEMP; Idx++ )
      if( BIT_TST(Tele.AvailMsk, ID_TEMP1+Idx) )
        JetiEx.SetSensorValue( ID_TEMP1+Idx, SensTemp[Idx]->TValTemp );

    #if _SENS_GPS_ == 1
    // GPS
    // ---
    if( BIT_TST(Tele.AvailMsk, ID_GPS_SATS) )
      JetiEx.SetSensorValue( ID_GPS_SATS, SensGps->TVal.Sats );
    if( BIT_TST(Tele.AvailMsk, ID_GPS_LAT) )
      JetiEx.SetSensorValueGPS( ID_GPS_LAT, false, SensGps->TVal.Latitude );
    if( BIT_TST(Tele.AvailMsk, ID_GPS_LON) )
      JetiEx.SetSensorValueGPS( ID_GPS_LON, true, SensGps->TVal.Longitude );
    if( BIT_TST(Tele.AvailMsk, ID_GPS_ALT) )
      JetiEx.SetSensorValue( ID_GPS_ALT, SensGps->TVal.Altitude );
    if( BIT_TST(Tele.AvailMsk, ID_GPS_SPEED) )
      JetiEx.SetSensorValue( ID_GPS_SPEED, SensGps->TVal.Speed );
    if( BIT_TST(Tele.AvailMsk, ID_GPS_SPEED_MAX) )
      JetiEx.SetSensorValue( ID_GPS_SPEED_MAX, SensGps->TVal.SpeedMax );
    if( BIT_TST(Tele.AvailMsk, ID_GPS_HEAD) )
      JetiEx.SetSensorValue( ID_GPS_HEAD, SensGps->TVal.Heading );
    if( BIT_TST(Tele.AvailMsk, ID_GPS_COURSE) )
      JetiEx.SetSensorValue( ID_GPS_COURSE, SensGps->TVal.CourseTo );
    if( BIT_TST(Tele.AvailMsk, ID_GPS_DIST_TO) )
      JetiEx.SetSensorValue( ID_GPS_DIST_TO, SensGps->TVal.DistTo );
    if( BIT_TST(Tele.AvailMsk, ID_GPS_DIST_TRIP) )
      JetiEx.SetSensorValue( ID_GPS_DIST_TRIP, SensGps->TVal.DistTrip );
    if( BIT_TST(Tele.AvailMsk, ID_GPS_DIST_TOT) )
      JetiEx.SetSensorValue( ID_GPS_DIST_TOT, SensGps->TVal.DistTotDez2 );
    #endif

    #if _SENS_ACC_ == 1
    // Beschleunigung / 3D Raum
    // ------------------------
    if( BIT_TST(Tele.AvailMsk, ID_ACC_GFORCE_X) )
      JetiEx.SetSensorValue( ID_ACC_GFORCE_X, SensAcc->TVal.GForceX );
    if( BIT_TST(Tele.AvailMsk, ID_ACC_GFORCE_Y) )
      JetiEx.SetSensorValue( ID_ACC_GFORCE_Y, SensAcc->TVal.GForceY );
    if( BIT_TST(Tele.AvailMsk, ID_ACC_GFORCE_Z) )
      JetiEx.SetSensorValue( ID_ACC_GFORCE_Z, SensAcc->TVal.GForceZ );
    if( BIT_TST(Tele.AvailMsk, ID_ACC_PITCH) )
      JetiEx.SetSensorValue( ID_ACC_PITCH, SensAcc->TVal.Pitch );
    if( BIT_TST(Tele.AvailMsk, ID_ACC_ROLL) )
      JetiEx.SetSensorValue( ID_ACC_ROLL, SensAcc->TVal.Roll );
    #endif
    }
}


// ===================
// EEProm read / write
// ===================
void EEpromRW( bool Wr )
{
  uint8_t Idx = sizeof( EEprom );
  uint8_t *Pos = (uint8_t *)&EEprom;

  if( Wr )
    EEpromChk();
  while ( Idx-- )
    {
    if( Wr )
      {
      //noInterrupts();           // write muss kontinuierlich erfolgen, steuert angeblich die Lib selber
      EEPROM.update( Idx, *Pos ); // nur bei Abweichung schreiben
      //interrupts();
      }
    else
      *Pos = EEPROM.read( Idx );
    Pos++;
    }
  if( !Wr )
    EEpromChk();
}

// EEProm gültig ?
// ===============
void EEpromChk( void )
{
  if( EEprom.NV.Magic != EEP_MAGIC_NV )         // Fabrikneu ?
    {
    memset( &EEprom.NV, 0, sizeof(EEprom.NV) ); // Summenwerte auch initialisieren
    EEprom.NV.Magic = EEP_MAGIC_NV;
    }

  if( EEprom.Data.Magic != EEP_MAGIC_DATA )
    {
    memset( &EEprom.Data, 0, sizeof(EEprom.Data) );
    EEprom.Data.Magic = EEP_MAGIC_DATA;
    // default Werte setzen
    EEprom.Data.SensActivMsk     = SENS_MSK_DEF;       // aktive Sensoren
    EEprom.Data.RpmCfgAccuracy   = RPM_ACCURACY_DEF;   // RPM Anzeige, Genauigkeit n UPM
    EEprom.Data.FuelCfgVolume    = FUEL_VOLUME_DEF;    // Tankvolumen
    EEprom.Data.FuelCfgFlowCnt1L = FUEL_FLOWCNT_1L;    // # Pulse / Liter
    }
}









